from typing import List

from pydantic import BaseModel


class AssociationEntry(BaseModel):
    name: str = None
    category: str = None
    mjdobs: float = None

    @classmethod
    def from_dict(cls, d):
        return cls(name=d['name'],
                   category=d['category'],
                   mjdobs=d['mjdobs'])

    class Config:
        schema_extra = {
            "example": {
                "name": "/some/path/file.fits",
                "category": "BIAS",
                "mjdobs": 56292.5
            }
        }

    def __repr__(self):
        return f'{self.name} {self.category} {self.mjdobs}'

    def __str__(self):
        return repr(self)


class AssociationResultDTO(BaseModel):
    files: List[AssociationEntry] = None
    jobs: List[AssociationEntry] = None
    complete: bool = None
    optional: bool = None
    level: float = None
    task_name: str = None

    @classmethod
    def from_dict(cls, d):
        return cls(
            files=[AssociationEntry.from_dict(f) for f in d.get('files', [])],
            jobs=[AssociationEntry.from_dict(f) for f in d.get('jobs', [])],
            complete=d['complete'],
            optional=d['optional'],
            level=d['level'],
            task_name=d['task_name'],
        )

    class Config:
        schema_extra = {
            "example": {
                "files": [AssociationEntry.Config.schema_extra['example']],
                "jobs": [AssociationEntry.Config.schema_extra['example']],
                "complete": True,
                "optional": False,
                "level": 1,
                "task_name": "bias",
            }
        }

    def __repr__(self):
        result = [f'Association name: {self.task_name}',
                  f'Association level: {self.level}',
                  f'Complete: {self.complete}',
                  f'Optional: {self.optional}']
        for f in self.files:
            result.append(str(f))
        for job in self.jobs:
            result.append(str(job))
        return '\n'.join(result)

    def __str__(self):
        return repr(self)
