from typing import List, Dict

from pydantic import BaseModel

from edps.client.AssociationResult import AssociationResultDTO
from edps.client.FitsFile import FitsFile

Setup = Dict[str, str]
Header = Dict[str, str]


class JobSummary(BaseModel):
    job_id: str = None
    input_files: List[FitsFile] = None
    input_job_ids: List[str] = None
    instrument: str = None
    mjdobs: float = None
    task_name: str = None
    assoc_level: float = None
    task_id: str = None
    association_details: List[AssociationResultDTO] = None
    setup: Setup = None
    header: Header = None
    workflow_names: List[str] = None

    @classmethod
    def from_dict(cls, d: Dict) -> 'JobSummary':
        return cls(
            job_id=d['job_id'],
            input_files=[FitsFile.from_dict(x) for x in d.get('input_files', [])],
            input_job_ids=d.get('input_job_ids', []),
            instrument=d['instrument'],
            mjdobs=d['mjdobs'],
            task_name=d['task_name'],
            assoc_level=d['assoc_level'],
            task_id=d['task_id'],
            association_details=[AssociationResultDTO.from_dict(x) for x in d.get('association_details', [])],
            setup=d.get('setup', {}),
            header=d.get('header', {}),
            workflow_names=d.get('workflow_names', []),
        )

    class Config:
        schema_extra = {
            "example": {
                "job_id": "5c04c6ce-5dd0-4a32-a9b8-41b55f66319c",
                "input_files": [FitsFile.Config.schema_extra['example']],
                "input_job_ids": ["6c04c6ce-5dd0-4a32-a9b8-41b55f66319c"],
                "instrument": "FORS1",
                "mjdobs": 508346.93563551357,
                "task_name": "bias",
                "assoc_level": 0,
                "task_id": "bias.fors_bias",
                "association_details": [AssociationResultDTO.Config.schema_extra['example']],
                "setup": {"det.binx": "1", "det.biny": "1"},
                "header": {"tpl.start": "xxx", "tpl.id": "yyy", "tpl.name": "zzz"},
                "workflow_names": ["edps.workflow.fors_imaging_wkf", "edps.workflow.kmos"],
            }
        }
