from typing import List

from edps.client.FitsFile import FitsFile


class ProductFilter(object):
    def is_accepted(self, file: FitsFile) -> bool:
        raise NotImplementedError


class FilterFactory(object):
    @staticmethod
    def create_filter(input_filter: List[str], mode: str) -> ProductFilter:
        if len(input_filter) == 0:
            return AcceptAllFilter()
        elif mode.upper() == 'REJECT':
            return RejectFilter(input_filter)
        else:
            return SelectFilter(input_filter)


class AcceptAllFilter(ProductFilter):
    def is_accepted(self, file: FitsFile) -> bool:
        return True


class SelectFilter(ProductFilter):
    def __init__(self, input_filter: List[str]):
        self.input_filter = set(input_filter)

    def is_accepted(self, file: FitsFile) -> bool:
        return file.category in self.input_filter


class RejectFilter(ProductFilter):
    def __init__(self, input_filter: List[str]):
        self.input_filter = set(input_filter)

    def is_accepted(self, file: FitsFile) -> bool:
        return file.category not in self.input_filter
